package com.qmth.themis.exam.listener.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.google.gson.Gson;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.WebsocketDto;
import com.qmth.themis.business.entity.TMRocketMessage;
import com.qmth.themis.business.enums.MqEnum;
import com.qmth.themis.business.service.TMRocketMessageService;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.exam.listener.service.MqOeLogicService;
import com.qmth.themis.exam.websocket.WebSocketOeServer;
import com.qmth.themis.mq.dto.MqDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.lang.reflect.InvocationTargetException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Description: mq执行逻辑
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/31
 */
@Service
public class MqOeLogicServiceImpl implements MqOeLogicService {
    private final static Logger log = LoggerFactory.getLogger(MqOeLogicServiceImpl.class);

    @Resource
    RedisUtil redisUtil;

    @Resource
    TMRocketMessageService tmRocketMessageService;

    /**
     * mq最大重试次数逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqMaxReconsumeTime(MqDto mqDto, String key) {
        //超过最大重试次数，保存到数据库，后续可以发短信通知系统管理人员
        Gson gson = new Gson();
        mqDto.setAck(SystemConstant.POSION_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * oe逻辑
     *
     * @param mqDto
     * @param key
     * @throws ClassNotFoundException
     * @throws IllegalAccessException
     * @throws InstantiationException
     * @throws NoSuchMethodException
     * @throws InvocationTargetException
     */
    @Override
    @Transactional
    public void execMqOqLogic(MqDto mqDto, String key) throws ClassNotFoundException, IllegalAccessException, InstantiationException, NoSuchMethodException, InvocationTargetException {
        Gson gson = new Gson();
        MqEnum mqEnum = mqDto.getType();
        ConcurrentHashMap<String, WebSocketOeServer> webSocketMap = WebSocketOeServer.getWebSocketMap();
        if (MqEnum.WEBSOCKET_OFFLINE_LOG.ordinal() == mqEnum.ordinal()) {//下线
            JSONArray jsonArray = JSONArray.parseArray(String.valueOf(mqDto.getBody()));
            Set<String> examStudentIdentitySet = jsonArray.toJavaObject(Set.class);
            log.info("examStudentIdentitySet:{}", JacksonUtil.parseJson(examStudentIdentitySet));
            webSocketMap.forEach((k, v) -> {
                examStudentIdentitySet.forEach(s -> {
                    if (k.contains(s)) {
                        Map map = new HashMap<>();
                        map.put("offLineId", k);
                        WebsocketDto websocketDto = new WebsocketDto("offLine", map);
                        v.sendMessage(websocketDto);
                    }
                });
            });
        } else if (MqEnum.WEBSOCKET_IM_CLUSTERING_LOG.ordinal() == mqEnum.ordinal()) {//点对点消息
            JSONArray jsonArray = JSONArray.parseArray(String.valueOf(mqDto.getBody()));
            Set<String> examStudentIdentitySet = jsonArray.toJavaObject(Set.class);
            log.info("examStudentIdentitySet:{}", JacksonUtil.parseJson(examStudentIdentitySet));
            webSocketMap.forEach((k, v) -> {
                examStudentIdentitySet.forEach(s -> {
                    if (k.contains(s)) {
                        Map map = new HashMap<>();
                        map.put("message", k);
                        WebsocketDto websocketDto = new WebsocketDto("message", map);
                        v.sendMessage(websocketDto);
                        return;
                    }
                });
            });
        } else if (MqEnum.WEBSOCKET_IM_BROADCASTING_LOG.ordinal() == mqEnum.ordinal()) {//广播消息
            JSONArray jsonArray = JSONArray.parseArray(String.valueOf(mqDto.getBody()));
            Set<String> examStudentIdentitySet = jsonArray.toJavaObject(Set.class);
            log.info("examStudentIdentitySet:{}", JacksonUtil.parseJson(examStudentIdentitySet));
            webSocketMap.forEach((k, v) -> {
                examStudentIdentitySet.forEach(s -> {
                    if (k.contains(s)) {
                        Map map = new HashMap<>();
                        map.put("message", k);
                        WebsocketDto websocketDto = new WebsocketDto("message", map);
                        v.sendMessage(websocketDto);
                    }
                });
            });
        }
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }
}
