package com.qmth.themis.exam.websocketTemplete;

import com.alibaba.fastjson.JSONObject;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.WebsocketDto;
import com.qmth.themis.business.entity.TOeExamRecord;
import com.qmth.themis.business.service.TOeExamRecordService;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 考生端websocket模版
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/24
 */
@Component
public class WebSocketOeMessageTemplete {
    private final static Logger log = LoggerFactory.getLogger(WebSocketOeMessageTemplete.class);

    @Resource
    TOeExamRecordService tOeExamRecordService;

    /**
     * 客户端已下载试卷
     *
     * @param body
     * @return
     */
    @Transactional
    public WebsocketDto clientPaperDownload(String body) {
        JSONObject jsonObject = JSONObject.parseObject(body);
        log.info("clientPaperDownload jsonObject:{}", jsonObject.toJSONString());
        if (Objects.isNull(jsonObject.get("recordId")) || Objects.equals(jsonObject.get("recordId"), "")) {
            throw new BusinessException(ExceptionResultEnum.RECORD_ID_IS_NULL);
        }
        Long recordId = Long.parseLong(String.valueOf(jsonObject.get("recordId")));
        TOeExamRecord tOeExamRecord = tOeExamRecordService.getById(recordId);
        if (Objects.isNull(tOeExamRecord)) {
            throw new BusinessException(ExceptionResultEnum.RECORD_NO);
        }
        tOeExamRecord.setPaperDownload(0);
        tOeExamRecordService.updateById(tOeExamRecord);
        //todo 更新redis考试记录缓存
        return new WebsocketDto(SystemConstant.SUCCESS);
    }

    /**
     * 状态同步
     *
     * @param body
     * @return
     */
    @Transactional
    public WebsocketDto syncStatus(String body) {
        JSONObject jsonObject = JSONObject.parseObject(body);
        log.info("syncStatus jsonObject:{}", jsonObject.toJSONString());
        if (Objects.isNull(jsonObject.get("recordId")) || Objects.equals(jsonObject.get("recordId"), "")) {
            throw new BusinessException(ExceptionResultEnum.RECORD_ID_IS_NULL);
        }
        Long recordId = Long.parseLong(String.valueOf(jsonObject.get("recordId")));
        if (Objects.isNull(jsonObject.get("progress")) || Objects.equals(jsonObject.get("progress"), "")) {
            throw new BusinessException("答题进度不能为空");
        }
        Double progress = Double.parseDouble(String.valueOf(jsonObject.get("durationSeconds")));
        if (Objects.isNull(jsonObject.get("durationSeconds")) || Objects.equals(jsonObject.get("durationSeconds"), "")) {
            throw new BusinessException("考试累计用时秒数不能为空");
        }
        Integer durationSeconds = Integer.parseInt((String.valueOf(jsonObject.get("durationSeconds"))));
        TOeExamRecord tOeExamRecord = tOeExamRecordService.getById(recordId);
        if (Objects.isNull(tOeExamRecord)) {
            throw new BusinessException(ExceptionResultEnum.RECORD_NO);
        }
        tOeExamRecord.setAnswerProgress(progress);
        tOeExamRecord.setDurationSeconds(durationSeconds);
        tOeExamRecordService.updateById(tOeExamRecord);
        return this.syncAck();
    }

    /**
     * 同步确认
     *
     * @return
     */
    public WebsocketDto syncAck() {
        Map map = new HashMap<>();
        map.put(SystemConstant.ACK_MESSAGE, System.currentTimeMillis());
        return new WebsocketDto(Thread.currentThread().getStackTrace()[1].getMethodName(), map);
    }

    /**
     * 监考强制活体验证
     *
     * @param body
     * @return
     */
    public WebsocketDto invigilateLivenessVerify(String body) {
        return null;
    }

    /**
     * 监考消息
     *
     * @param body
     * @return
     */
    public WebsocketDto invigilateNotice(String body) {
        return null;
    }

    /**
     * 监考消息确认
     *
     * @return
     */
    public WebsocketDto invigilateNoticeAck() {
        Map map = new HashMap<>();
        map.put(SystemConstant.ACK_MESSAGE, System.currentTimeMillis());
        return new WebsocketDto(this.getClass().getEnclosingMethod().getName(), map);
    }

    /**
     * 监考强制收卷
     *
     * @param body
     * @return
     */
    public WebsocketDto invigilateStopExam(String body) {
        return null;
    }
}
